# coding=utf-8
# Copyright 2022 EleutherAI and the HuggingFace Inc. team. All rights reserved.
#
# This code is based on EleutherAI's GPT-NeoX library and the GPT-NeoX
# and OPT implementations in this library. It has been modified from its
# original forms to accommodate minor architectural differences compared
# to GPT-NeoX and OPT used by the Meta AI team that trained the model.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" PyTorch LLaMA model."""
import math
import numpy as np
from typing import List, Optional, Tuple, Union

import torch
import torch.nn.functional as F
import torch.utils.checkpoint
from torch import nn
from torch.nn import BCEWithLogitsLoss, CrossEntropyLoss, MSELoss

from transformers.activations import ACT2FN
from transformers.modeling_outputs import BaseModelOutputWithPast, CausalLMOutputWithPast, SequenceClassifierOutputWithPast
from transformers.modeling_utils import PreTrainedModel
from transformers.utils import add_start_docstrings, add_start_docstrings_to_model_forward, logging, replace_return_docstrings

from transformers.models.llama.configuration_llama import LlamaConfig


try:
    from flash_attn.flash_attn_interface import (
        flash_attn_func, 
        flash_attn_kvpacked_func, 
        flash_attn_qkvpacked_func,
        flash_attn_varlen_kvpacked_func, 
    )
    from flash_attn.bert_padding import unpad_input, pad_input
    flash_attn_v2_installed = True
    print('>>>> Flash Attention installed')
except ImportError:
    flash_attn_v2_installed = False
    raise ImportError('Please install Flash Attention: `pip install flash-attn --no-build-isolation`')

try:
    from flash_attn.layers.rotary import apply_rotary_emb_func
    flash_rope_installed = True
    print('>>>> Flash RoPE installed')
except ImportError:
    flash_rope_installed = False
    raise ImportError('Please install RoPE kernels: `pip install git+https://github.com/HazyResearch/flash-attention.git#subdirectory=csrc/rotary`')


logger = logging.get_logger(__name__)

_CONFIG_FOR_DOC = "LlamaConfig"


# Copied from transformers.models.bart.modeling_bart._make_causal_mask
def _make_causal_mask(
    input_ids_shape: torch.Size, dtype: torch.dtype, device: torch.device, past_key_values_length: int = 0
):
    """
    Make causal mask used for bi-directional self-attention.
    """
    bsz, tgt_len = input_ids_shape
    mask = torch.full((tgt_len, tgt_len), torch.finfo(dtype).min, device=device)
    mask_cond = torch.arange(mask.size(-1), device=device)
    mask.masked_fill_(mask_cond < (mask_cond + 1).view(mask.size(-1), 1), 0)
    mask = mask.to(dtype)

    if past_key_values_length > 0:
        mask = torch.cat([torch.zeros(tgt_len, past_key_values_length, dtype=dtype, device=device), mask], dim=-1)
    return mask[None, None, :, :].expand(bsz, 1, tgt_len, tgt_len + past_key_values_length)


# Copied from transformers.models.bart.modeling_bart._expand_mask
def _expand_mask(mask: torch.Tensor, dtype: torch.dtype, tgt_len: Optional[int] = None):
    """
    Expands attention_mask from `[bsz, seq_len]` to `[bsz, 1, tgt_seq_len, src_seq_len]`.
    """
    bsz, src_len = mask.size()
    tgt_len = tgt_len if tgt_len is not None else src_len

    expanded_mask = mask[:, None, None, :].expand(bsz, 1, tgt_len, src_len).to(dtype)

    inverted_mask = 1.0 - expanded_mask

    return inverted_mask.masked_fill(inverted_mask.to(torch.bool), torch.finfo(dtype).min)


#@torch.jit.script
def rmsnorm_func(hidden_states, weight, variance_epsilon):
    input_dtype = hidden_states.dtype
    hidden_states = hidden_states.to(torch.float32)
    variance = hidden_states.pow(2).mean(-1, keepdim=True)
    hidden_states = hidden_states * torch.rsqrt(variance + variance_epsilon)
    return (weight * hidden_states).to(input_dtype)
    

class LlamaRMSNorm(nn.Module):
    def __init__(self, hidden_size, eps=1e-6):
        """
        LlamaRMSNorm is equivalent to T5LayerNorm
        """
        super().__init__()
        self.weight = nn.Parameter(torch.ones(hidden_size))
        self.register_buffer(
            "variance_epsilon",
            torch.tensor(eps),
            persistent=False,
        )
        
    def forward(self, hidden_states):
        return rmsnorm_func(hidden_states, self.weight, self.variance_epsilon)


class LlamaRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, device=None, enable_head_metrics=False):
        super().__init__()

        self.dim = dim
        self.max_position_embeddings = max_position_embeddings
        self.base = base
        inv_freq = 1.0 / (self.base ** (torch.arange(0, self.dim, 2).float().to(device) / self.dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        # Build here to make `torch.jit.trace` work.
        self._set_cos_sin_cache(
            seq_len=max_position_embeddings, device=self.inv_freq.device, 
            dtype=torch.get_default_dtype(), enable_head_metrics=enable_head_metrics
        )

    def _set_cos_sin_cache(self, seq_len, device, dtype, enable_head_metrics=False):
        self.max_seq_len_cached = seq_len
        # t = torch.arange(self.max_seq_len_cached, device=device, dtype=self.inv_freq.dtype)
        t = np.arange(self.max_seq_len_cached, dtype=np.float64)
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        # freqs = torch.einsum("i,j->ij", t, self.inv_freq)
        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)
        self.register_buffer("cos_cached", emb.cos()[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", emb.sin()[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None, enable_head_metrics=False):
        # x: [bs, num_attention_heads, seq_len, head_size]
        if seq_len > self.max_seq_len_cached:
            self._set_cos_sin_cache(seq_len=seq_len, device=x.device, dtype=x.dtype, enable_head_metrics=enable_head_metrics)

        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinearScalingRotaryEmbedding(LlamaRotaryEmbedding):
    """LlamaRotaryEmbedding extended with linear scaling. Credits to the Reddit user /u/kaiokendev"""

    def __init__(self, dim, max_position_embeddings=2048, base=10000, device=None, scaling_factor=1.0, enable_head_metrics=False):
        self.scaling_factor = scaling_factor
        super().__init__(dim, max_position_embeddings, base, device, enable_head_metrics)

    def _set_cos_sin_cache(self, seq_len, device, dtype, enable_head_metrics=False):
        self.max_seq_len_cached = seq_len

        t = np.arange(self.max_seq_len_cached, dtype=np.float64)
        t = t / self.scaling_factor
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)

        if enable_head_metrics:
            ms_scale = torch.arange(1, 2, 1/32).unsqueeze(1).unsqueeze(2)  # [num_heads, 1, 1]
            emb_cos = torch.mul(ms_scale, emb.cos().repeat(32, 1, 1))  # [num_heads, seq_len, dim]
            emb_sin = torch.mul(ms_scale, emb.sin().repeat(32, 1, 1))
            self.register_buffer("cos_cached", emb_cos[None, :, :, :].to(dtype), persistent=False)  # [1, num_heads, seq_len, dim]
            self.register_buffer("sin_cached", emb_sin[None, :, :, :].to(dtype), persistent=False)

        else:
            self.register_buffer("cos_cached", emb.cos()[None, None, :, :].to(dtype), persistent=False)  # [1, 1, seq_len, dim]
            self.register_buffer("sin_cached", emb.sin()[None, None, :, :].to(dtype), persistent=False)


class LlamaLinear_1ScalingRotaryEmbedding(LlamaRotaryEmbedding):
    """LlamaRotaryEmbedding extended with linear scaling. Credits to the Reddit user /u/kaiokendev"""

    def __init__(self, dim, max_position_embeddings=2048, base=10000, device=None, scaling_factor=1.0, enable_head_metrics=False):
        self.scaling_factor = scaling_factor
        super().__init__(dim, max_position_embeddings, base, device, enable_head_metrics)

    def _set_cos_sin_cache(self, seq_len, device, dtype, enable_head_metrics=False):
        self.max_seq_len_cached = seq_len

        t = np.arange(self.max_seq_len_cached, dtype=np.float64)
        t = t / self.scaling_factor
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)

        mscale = float(_yarn_get_mscale(self.scaling_factor))
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)


# Inverse dim formula to find dim based on number of rotations
def _yarn_find_correction_dim(num_rotations, dim, base=10000, max_position_embeddings=2048):
    return (dim * math.log(max_position_embeddings/(num_rotations * 2 * math.pi)))/(2 * math.log(base))

# Find dim range bounds based on rotations
def _yarn_find_correction_range(low_rot, high_rot, dim, base=10000, max_position_embeddings=2048):
    low = math.floor(_yarn_find_correction_dim(
        low_rot, dim, base, max_position_embeddings))
    high = math.ceil(_yarn_find_correction_dim(
        high_rot, dim, base, max_position_embeddings))
    return max(low, 0), min(high, dim-1)  # Clamp values just in case

def _yarn_linear_ramp_mask(min, max, dim):
    if min == max:
        max += 0.001  # Prevent singularity

    linear_func = (torch.arange(dim, dtype=torch.float32) - min) / (max - min)
    ramp_func = torch.clamp(linear_func, 0, 1)
    return ramp_func

def _yarn_get_mscale(scale=1):
    if scale <= 1:
        return 1.0
    return 0.1 * math.log(scale) + 1.0


class LlamaYaRNScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, extrapolation_factor=1, attn_factor=1, beta_fast=32, beta_slow=1, scaling_type=None, device=None):
        super().__init__()

        self.dim = dim
        self.max_position_embeddings = max_position_embeddings
        self.base = base
        self.scale = scale
        self.original_max_position_embeddings = original_max_position_embeddings
        self.extrapolation_factor = extrapolation_factor
        self.attn_factor = attn_factor
        self.beta_fast = beta_fast
        self.beta_slow = beta_slow

        if scaling_type == 'yarn_old':
            self.yarn(device)
        elif scaling_type == 'yarn':
            self.revised_yarn(device)

        # Build here to make `torch.jit.trace` work.
        self.max_seq_len_cached = max_position_embeddings

        t = np.arange(self.max_seq_len_cached, dtype=np.float64)
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)
        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))

        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)
        dtype = torch.get_default_dtype()

        self.register_buffer("cos_cached", (emb.cos() * self.mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * self.mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        # This `if` block is unlikely to be run after we build sin/cos in `__init__`. Keep the logic here just in case.
        if seq_len > self.max_seq_len_cached:
            self.max_seq_len_cached = seq_len

            t = torch.arange(self.max_seq_len_cached, device=x.device, dtype=self.inv_freq.dtype)
            freqs = torch.einsum("i,j->ij", t, self.inv_freq)
            # Different from paper, but it uses a different permutation in order to obtain the same calculation
            emb = torch.cat((freqs, freqs), dim=-1).to(x.device)

            self.register_buffer("cos_cached", (emb.cos() * self.mscale)[None, None, :, :].to(x.dtype), persistent=False)
            self.register_buffer("sin_cached", (emb.sin() * self.mscale)[None, None, :, :].to(x.dtype), persistent=False)
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )

    def yarn(self, device):
        pos_freqs = self.base ** (torch.arange(0, self.dim, 2).float().to(device) / self.dim)
        inv_freq_extrapolation = 1.0 / pos_freqs
        inv_freq_interpolation = 1.0 / (self.scale * pos_freqs)

        low, high = _yarn_find_correction_range(self.beta_fast, self.beta_slow, self.dim, self.base, self.original_max_position_embeddings)
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(low, high, self.dim // 2).float().to(device)) * self.extrapolation_factor # Get n-d rotational scaling corrected for extrapolation
        inv_freq = inv_freq_interpolation * (1 - inv_freq_mask) + inv_freq_extrapolation * inv_freq_mask

        self.register_buffer("inv_freq", inv_freq, persistent=False)
        self.mscale = float(_yarn_get_mscale(self.scale) * self.attn_factor) # Get n-d magnitude scaling corrected for interpolation

    def revised_yarn(self, device):
        inv_freq = 1.0 / (self.base ** (torch.arange(0, self.dim, 2).float().to(device) / self.dim))

        low, high = _yarn_find_correction_range(self.beta_fast, self.beta_slow, self.dim, self.base, self.original_max_position_embeddings)
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(low, high, self.dim // 2).float().to(device)) * self.extrapolation_factor

        inv_freq = inv_freq / ((1-inv_freq_mask)*self.scale + inv_freq_mask)
        self.register_buffer("inv_freq", inv_freq, persistent=False)
        self.mscale = float(_yarn_get_mscale(self.scale) * self.attn_factor)


class LlamaVanillaNTKScalingRotaryEmbedding(LlamaRotaryEmbedding):

    def __init__(self, dim, max_position_embeddings=2048, base=10000, device=None, scaling_factor=1.0, enable_head_metrics=False):
        self.scaling_factor = scaling_factor
        super().__init__(dim, max_position_embeddings, base, device, enable_head_metrics)

    def _set_cos_sin_cache(self, seq_len, device, dtype, enable_head_metrics=False):
        self.max_seq_len_cached = seq_len

        base = self.base * self.scaling_factor ** (self.dim / (self.dim - 2))
        inv_freq = 1.0 / (base ** (torch.arange(0, self.dim, 2).float().to(device) / self.dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = np.arange(self.max_seq_len_cached, dtype=np.float64)
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        # freqs = torch.einsum("i,j->ij", t, self.inv_freq)
        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)
        self.register_buffer("cos_cached", emb.cos()[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", emb.sin()[None, None, :, :].to(dtype), persistent=False)


class LlamaLinYarnScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, extrapolation_factor=1, attn_factor=1, beta_fast=32, beta_slow=1, num_heads=32, device=None):
        super().__init__()

        self.dim = dim
        self.base = base
        self.original_max_position_embeddings = original_max_position_embeddings
        self.extrapolation_factor = extrapolation_factor
        self.attn_factor = attn_factor
        self.beta_fast = beta_fast
        self.beta_slow = beta_slow
        self.device = device
        
        chunks_head = num_heads // 4  # Linear, Yarn, Linear*Yarn, Yarn*Linear

        # Build here to make `torch.jit.trace` work.
        self.max_seq_len_cached = max_position_embeddings

        t = torch.arange(self.max_seq_len_cached, device=device, dtype=torch.float64)
        inv_freq = 1.0 / (base ** (torch.arange(0, self.dim, 2).float().to(device) / self.dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        # [chunks_head, seq_len, dim]
        cos_emb_1, sin_emb_1 = self._get_linear_yarn_emb(t, inv_freq, chunks_head, linear_scale=scale, yarn_scale=1)
        cos_emb_2, sin_emb_2 = self._get_linear_yarn_emb(t, inv_freq, chunks_head, linear_scale=1, yarn_scale=scale)
        
        linear_scale = scale // 2
        yarn_scale = scale // linear_scale
        cos_emb_3, sin_emb_3 = self._get_linear_yarn_emb(t, inv_freq, chunks_head, linear_scale, yarn_scale)
        cos_emb_4, sin_emb_4 = self._get_linear_yarn_emb(t, inv_freq, chunks_head, yarn_scale, linear_scale)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2, cos_emb_3, cos_emb_4], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2, sin_emb_3, sin_emb_4], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def _get_linear_yarn_emb(self, t, inv_freq, chunks_head, linear_scale, yarn_scale):
        t = t / linear_scale

        inv_freq, mscale = self.revised_yarn(self.device, inv_freq, yarn_scale)

        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))

        # Different from paper, but it uses a different permutation in order to obtain the same calculation
        emb = torch.cat((freqs, freqs), dim=-1)

        cos_emb = (emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb = (emb.sin() * mscale).repeat(chunks_head, 1, 1)

        return cos_emb, sin_emb

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        # This `if` block is unlikely to be run after we build sin/cos in `__init__`. Keep the logic here just in case.
        if seq_len > self.max_seq_len_cached:
            self.max_seq_len_cached = seq_len

            t = torch.arange(self.max_seq_len_cached, device=x.device, dtype=self.inv_freq.dtype)
            freqs = torch.einsum("i,j->ij", t, self.inv_freq)
            # Different from paper, but it uses a different permutation in order to obtain the same calculation
            emb = torch.cat((freqs, freqs), dim=-1).to(x.device)

            self.register_buffer("cos_cached", (emb.cos() * self.mscale)[None, None, :, :].to(x.dtype), persistent=False)
            self.register_buffer("sin_cached", (emb.sin() * self.mscale)[None, None, :, :].to(x.dtype), persistent=False)
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )

    def revised_yarn(self, device, inv_freq, yarn_scale):
        low, high = _yarn_find_correction_range(self.beta_fast, self.beta_slow, self.dim, self.base, self.original_max_position_embeddings)
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(low, high, self.dim // 2).float().to(device)) * self.extrapolation_factor

        yarn_inv_freq = inv_freq / ((1-inv_freq_mask) * yarn_scale + inv_freq_mask)
        self.register_buffer("inv_freq", inv_freq, persistent=False)
        yarn_mscale = float(_yarn_get_mscale(yarn_scale) * self.attn_factor)

        return yarn_inv_freq, yarn_mscale


class LlamaLinYarn2ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        constant_dim = math.ceil(_yarn_find_correction_dim(num_rotations=128, dim=dim, base=base, max_position_embeddings=original_max_position_embeddings))
        ntk_dim = math.floor(_yarn_find_correction_dim(num_rotations=1, dim=dim, base=base, max_position_embeddings=original_max_position_embeddings))

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)

        # constant
        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)
        constant_freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))[:, :constant_dim]
        
        # linear
        linear_freqs = torch.outer(t / scale, self.inv_freq.to(device=t.device).to(t.dtype))[:, constant_dim:ntk_dim]

        # ntk
        ntk_base = base * (scale ** (dim / (dim - 2)))
        ntk_inv_freq = 1.0 / (ntk_base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        ntk_freqs = torch.outer(t, ntk_inv_freq.to(device=t.device).to(t.dtype))[:, ntk_dim:]

        freqs = torch.cat([constant_freqs, linear_freqs, ntk_freqs], dim=-1)
        emb = torch.cat((freqs, freqs), dim=-1)

        dtype = torch.get_default_dtype()
        mscale = 1  # float(_yarn_get_mscale(scale))

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn3ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        ntk_dim = math.floor(_yarn_find_correction_dim(num_rotations=1, dim=dim, base=base, max_position_embeddings=original_max_position_embeddings))

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)

        # my_ntk
        ntk_base = base * (scale ** (dim / (2 * ntk_dim)))
        inv_freq = 1.0 / (ntk_base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)

        dtype = torch.get_default_dtype()
        mscale = 1  # float(_yarn_get_mscale(scale))

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn4ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        mscale = 1  # float(_yarn_get_mscale(scale))
        chunks_head = num_heads // 2  # Linear, NTK

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        # linear
        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = np.arange(max_seq_len_cached, dtype=np.float64)
        t = t / scale
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)

        # [chunks_head, seq_len, dim]
        cos_emb_1 = (emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_1 = (emb.sin() * mscale).repeat(chunks_head, 1, 1)

        # ntk
        ntk_base = base * (scale ** (dim / (dim - 2)))
        ntk_inv_freq = 1.0 / (ntk_base ** (torch.arange(0, dim, 2).float().to(device) / dim))

        ntk_t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        ntk_freqs = torch.outer(ntk_t, ntk_inv_freq.to(device=t.device).to(t.dtype))
        
        ntk_emb = torch.cat((ntk_freqs, ntk_freqs), dim=-1)

        # [chunks_head, seq_len, dim]
        cos_emb_2 = (ntk_emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_2 = (ntk_emb.sin() * mscale).repeat(chunks_head, 1, 1)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn5ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        ntk_dim = math.floor(_yarn_find_correction_dim(num_rotations=1, dim=dim, base=base, max_position_embeddings=original_max_position_embeddings))

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        
        # linear
        mask = torch.clamp(torch.arange(dim // 2, dtype=torch.float32) / ntk_dim, 0, 1)
        mask_scale = mask * scale + (1 - mask)

        inv_freq = (1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))) / mask_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        linear_freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))[:, :ntk_dim]

        # ntk
        ntk_base = base * (scale ** (dim / (dim - 2)))
        ntk_inv_freq = 1.0 / (ntk_base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        ntk_freqs = torch.outer(t, ntk_inv_freq.to(device=t.device).to(t.dtype))[:, ntk_dim:]

        freqs = torch.cat([linear_freqs, ntk_freqs], dim=-1)
        emb = torch.cat((freqs, freqs), dim=-1)

        dtype = torch.get_default_dtype()
        mscale = 1  # float(_yarn_get_mscale(scale))

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn6ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, beta_fast=32, beta_slow=1, num_heads=32, device=None):
        super().__init__()

        mscale = 1  # float(_yarn_get_mscale(scale))
        chunks_head = num_heads // 2  # Linear, yarn

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        # linear
        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = np.arange(max_seq_len_cached, dtype=np.float64)
        t = t / scale
        t = torch.tensor(t, device=self.inv_freq.device, dtype=torch.float64)

        freqs = torch.outer(t, self.inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)

        # [chunks_head, seq_len, dim]
        cos_emb_1 = (emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_1 = (emb.sin() * mscale).repeat(chunks_head, 1, 1)

        # yarn
        low, high = _yarn_find_correction_range(beta_fast, beta_slow, dim, base, original_max_position_embeddings)
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(low, high, dim // 2).float().to(device))
        yarn_inv_freq = inv_freq / ((1-inv_freq_mask) * scale + inv_freq_mask)

        yarn_t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        yarn_freqs = torch.outer(yarn_t, yarn_inv_freq.to(device=t.device).to(t.dtype))
        yarn_emb = torch.cat((yarn_freqs, yarn_freqs), dim=-1)

        # [chunks_head, seq_len, dim]
        cos_emb_2 = (yarn_emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_2 = (yarn_emb.sin() * mscale).repeat(chunks_head, 1, 1)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn7ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        num_rotations = 2
        scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
        inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
        inv_freq /= inv_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)
        
        _mscale = 0.1 * torch.log(inv_scale + 1.0) + 1.0
        mscale = torch.cat((_mscale, _mscale), dim=-1)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn8ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        num_rotations = 2
        scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
        inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
        inv_freq /= inv_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)
        
        mscale = 0.1 * math.log(scale + 1.0) + 1.0

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )
    

class LlamaLinYarn9ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, num_heads=32, device=None):
        super().__init__()

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        num_rotations = 1
        scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
        inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
        inv_freq /= inv_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)
        
        mscale = 0.1 * math.log(scale + 1.0) + 1.0

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", (emb.cos() * mscale)[None, None, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", (emb.sin() * mscale)[None, None, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )
    

class LlamaLinYarn10ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, beta_fast=32, beta_slow=1, num_heads=32, device=None):
        super().__init__()

        chunks_head = num_heads // 2  # linyarn7, linyarn8

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        num_rotations = 2
        scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
        inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
        inv_freq /= inv_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)

        # linyarn7
        _mscale = 0.1 * torch.log(inv_scale + 1.0) + 1.0
        mscale1 = torch.cat((_mscale, _mscale), dim=-1)

        # [chunks_head, seq_len, dim]
        cos_emb_1 = (emb.cos() * mscale1).repeat(chunks_head, 1, 1)
        sin_emb_1 = (emb.sin() * mscale1).repeat(chunks_head, 1, 1)

        # linyarn8
        mscale2 = 0.1 * math.log(scale + 1.0) + 1.0

        # [chunks_head, seq_len, dim]
        cos_emb_2 = (emb.cos() * mscale2).repeat(chunks_head, 1, 1)
        sin_emb_2 = (emb.sin() * mscale2).repeat(chunks_head, 1, 1)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn11ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, beta_fast=32, beta_slow=1, num_heads=32, device=None):
        super().__init__()

        mscale = 0.1 * math.log(scale + 1.0) + 1.0
        chunks_head = num_heads // 2  # linyarn7, linyarn8

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings
        
        def get_emb_cache(num_rotations):
            scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

            inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
            inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
            inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
            inv_freq /= inv_scale
            self.register_buffer("inv_freq", inv_freq, persistent=False)

            t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
            freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
            emb = torch.cat((freqs, freqs), dim=-1)

            return emb

        # linyarn8
        linyarn8_emb = get_emb_cache(num_rotations=2)

        # [chunks_head, seq_len, dim]
        cos_emb_1 = (linyarn8_emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_1 = (linyarn8_emb.sin() * mscale).repeat(chunks_head, 1, 1)

        # linyarn9
        linyarn9_emb = get_emb_cache(num_rotations=1)

        # [chunks_head, seq_len, dim]
        cos_emb_2 = (linyarn9_emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_2 = (linyarn9_emb.sin() * mscale).repeat(chunks_head, 1, 1)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


class LlamaLinYarn12ScaledRotaryEmbedding(torch.nn.Module):
    def __init__(self, dim, max_position_embeddings=2048, base=10000, scale=1, original_max_position_embeddings=2048, beta_fast=32, beta_slow=1, num_heads=32, device=None):
        super().__init__()

        chunks_head = num_heads // 2  # linyarn8, linyarn8

        # Build here to make `torch.jit.trace` work.
        max_seq_len_cached = max_position_embeddings

        num_rotations = 2
        scale_dim = (dim * math.log(original_max_position_embeddings / (num_rotations * 2 * math.pi))) / (2 * math.log(base))

        inv_freq = 1.0 / (base ** (torch.arange(0, dim, 2).float().to(device) / dim))
        inv_freq_mask = (1 - _yarn_linear_ramp_mask(0, scale_dim, dim // 2).float().to(device))
        inv_scale = ((1 - inv_freq_mask) * scale + inv_freq_mask)
        inv_freq /= inv_scale
        self.register_buffer("inv_freq", inv_freq, persistent=False)

        t = torch.arange(max_seq_len_cached, device=device, dtype=torch.float64)
        freqs = torch.outer(t, inv_freq.to(device=t.device).to(t.dtype))
        emb = torch.cat((freqs, freqs), dim=-1)

        mscale = 0.1 * math.log(scale + 1.0) + 1.0

        # [chunks_head, seq_len, dim]
        cos_emb_1 = (emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_1 = (emb.sin() * mscale).repeat(chunks_head, 1, 1)

        # [chunks_head, seq_len, dim]
        cos_emb_2 = (emb.cos() * mscale).repeat(chunks_head, 1, 1)
        sin_emb_2 = (emb.sin() * mscale).repeat(chunks_head, 1, 1)

        cos_emb = torch.cat([cos_emb_1, cos_emb_2], dim=0)
        sin_emb = torch.cat([sin_emb_1, sin_emb_2], dim=0)

        dtype = torch.get_default_dtype()

        # [num_heads, seq_len, dim]
        self.register_buffer("cos_cached", cos_emb[None, :, :, :].to(dtype), persistent=False)
        self.register_buffer("sin_cached", sin_emb[None, :, :, :].to(dtype), persistent=False)

    def forward(self, x, seq_len=None):
        # x: [bs, num_attention_heads, seq_len, head_size]
        return (
            self.cos_cached[:, :, :, ...].to(dtype=x.dtype),
            self.sin_cached[:, :, :, ...].to(dtype=x.dtype),
        )


def rotate_half(x):
    """Rotates half the hidden dims of the input."""
    x1 = x[..., : x.shape[-1] // 2]
    x2 = x[..., x.shape[-1] // 2 :]
    return torch.cat((-x2, x1), dim=-1)


def apply_rotary_pos_emb(q, k, cos, sin, position_ids):
    if cos.size(0) != 1 and cos.size(1) != 1:
        # [bs, num_heads, seq_len, dim] -> [bs, seq_len, num_heads, dim]
        cos = torch.gather(
            cos, 2, position_ids.unsqueeze(1).unsqueeze(-1).expand(-1, cos.size(1), -1, cos.size(-1))
        ).transpose(1, 2)
        sin = torch.gather(
            sin, 2, position_ids.unsqueeze(1).unsqueeze(-1).expand(-1, sin.size(1), -1, sin.size(-1))
        ).transpose(1, 2)
    elif cos.size(0) == 1 and cos.size(1) != 1:
        # [num_heads, seq_len, dim] -> [bs, seq_len, num_heads, dim]
        cos = cos.squeeze().transpose(0, 1)[position_ids]
        sin = sin.squeeze().transpose(0, 1)[position_ids]
    else:
        cos = cos.squeeze()
        sin = sin.squeeze()
        # [bs, seq_len, dim] -> [bs, seq_len, 1, dim]
        cos = cos[position_ids].unsqueeze(2)
        sin = sin[position_ids].unsqueeze(2)

    q_embed = (q * cos) + (rotate_half(q) * sin)
    k_embed = (k * cos) + (rotate_half(k) * sin)
    return q_embed, k_embed


class LlamaMLP(nn.Module):
    def __init__(self, config):
        super().__init__()
        self.config = config
        self.hidden_size = config.hidden_size
        self.intermediate_size = config.intermediate_size
        self.gate_proj = nn.Linear(self.hidden_size, self.intermediate_size, bias=False)
        self.up_proj = nn.Linear(self.hidden_size, self.intermediate_size, bias=False)
        self.down_proj = nn.Linear(self.intermediate_size, self.hidden_size, bias=False)
        self.act_fn = ACT2FN[config.hidden_act]

    def forward(self, x):
        if self.config.pretraining_tp > 1:
            slice = self.intermediate_size // self.config.pretraining_tp
            gate_proj_slices = self.gate_proj.weight.split(slice, dim=0)
            up_proj_slices = self.up_proj.weight.split(slice, dim=0)
            down_proj_slices = self.down_proj.weight.split(slice, dim=1)

            gate_proj = torch.cat(
                [F.linear(x, gate_proj_slices[i]) for i in range(self.config.pretraining_tp)], dim=-1
            )
            up_proj = torch.cat([F.linear(x, up_proj_slices[i]) for i in range(self.config.pretraining_tp)], dim=-1)

            intermediate_states = (self.act_fn(gate_proj) * up_proj).split(slice, dim=2)
            down_proj = [
                F.linear(intermediate_states[i], down_proj_slices[i]) for i in range(self.config.pretraining_tp)
            ]
            down_proj = sum(down_proj)
        else:
            down_proj = self.down_proj(self.act_fn(self.gate_proj(x)) * self.up_proj(x))

        return down_proj

@torch.jit.script
def repeat_kv(hidden_states: torch.Tensor, n_rep: int) -> torch.Tensor:
    """
    This is the equivalent of torch.repeat_interleave(x, dim=1, repeats=n_rep). The hidden states go from (batch,
    num_key_value_heads, seqlen, head_dim) to (batch, num_attention_heads, seqlen, head_dim)
    """
    batch, slen, _, num_key_value_heads, head_dim = hidden_states.shape
    if n_rep == 1:
        return hidden_states
    hidden_states = hidden_states[:, :, :, :, None, :].expand(batch, slen, 2, num_key_value_heads, n_rep, head_dim)
    return hidden_states.reshape(batch, slen, 2, num_key_value_heads * n_rep, head_dim)


class LlamaAttention(nn.Module):
    """Multi-headed attention from 'Attention Is All You Need' paper"""

    def __init__(self, config: LlamaConfig):
        super().__init__()
        self.config = config
        self.hidden_size = config.hidden_size
        self.num_heads = config.num_attention_heads
        self.head_dim = self.hidden_size // self.num_heads
        self.num_key_value_heads = config.num_key_value_heads
        self.num_key_value_groups = self.num_heads // self.num_key_value_heads
        self.max_position_embeddings = config.max_position_embeddings
        self.rope_theta = 10000.0  # config.rope_theta

        self.enable_head_metrics = False  # False

        if (self.head_dim * self.num_heads) != self.hidden_size:
            raise ValueError(
                f"hidden_size must be divisible by num_heads (got `hidden_size`: {self.hidden_size}"
                f" and `num_heads`: {self.num_heads})."
            )
        self.q_proj = nn.Linear(self.hidden_size, self.num_heads * self.head_dim, bias=False)
        self.k_proj = nn.Linear(self.hidden_size, self.num_key_value_heads * self.head_dim, bias=False)
        self.v_proj = nn.Linear(self.hidden_size, self.num_key_value_heads * self.head_dim, bias=False)
        self.o_proj = nn.Linear(self.num_heads * self.head_dim, self.hidden_size, bias=False)

        self.register_buffer(
            "norm_factor",
            torch.sqrt(torch.tensor(self.head_dim, dtype=torch.float32)).to(torch.get_default_dtype()),
            persistent=False,
        )

        self._init_rope()

    def _init_rope(self):
        if self.config.rope_scaling is None:
            self.rotary_emb = LlamaRotaryEmbedding(self.head_dim, max_position_embeddings=self.max_position_embeddings)
        else:
            scaling_type = self.config.rope_scaling["type"]
            scaling_factor = self.config.rope_scaling["factor"]
            if scaling_type == "linear":
                self.rotary_emb = LlamaLinearScalingRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scaling_factor=scaling_factor, enable_head_metrics=self.enable_head_metrics
                )
            elif scaling_type == "linear_1":
                self.rotary_emb = LlamaLinear_1ScalingRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scaling_factor=scaling_factor, enable_head_metrics=self.enable_head_metrics
                )
            elif scaling_type == "ntk":
                self.rotary_emb = LlamaVanillaNTKScalingRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scaling_factor=scaling_factor, enable_head_metrics=self.enable_head_metrics
                )
            elif scaling_type == "yarn" or scaling_type == "yarn_old":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaYaRNScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    scaling_type=scaling_type
                )
            elif scaling_type == "linyarn":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarnScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn2":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn2ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn3":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn3ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn4":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn4ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn5":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn5ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn6":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn6ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn7":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn7ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn8":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn8ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn9":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn9ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn10":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn10ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn11":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn11ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            elif scaling_type == "linyarn12":
                original_max_position_embeddings = self.config.rope_scaling["original_max_position_embeddings"]
                self.rotary_emb = LlamaLinYarn12ScaledRotaryEmbedding(
                    self.head_dim, max_position_embeddings=self.max_position_embeddings, scale=scaling_factor, original_max_position_embeddings=original_max_position_embeddings,
                    num_heads = self.num_heads
                )
            else:
                raise ValueError(f"Unknown RoPE scaling type {scaling_type}")

    def _shape(self, tensor: torch.Tensor, seq_len: int, bsz: int):
        return tensor.view(bsz, seq_len, self.num_heads, self.head_dim).transpose(1, 2).contiguous()
    
    def _head_wise_statistics(self, query_states, key_states, q_len, kv_seq_len, bsz, attention_mask):
        query_states, key_states = query_states.transpose(1, 2), key_states.transpose(1, 2)
        attn_weights = torch.matmul(query_states, key_states.transpose(2, 3)) / math.sqrt(self.head_dim)

        if attn_weights.size() != (bsz, self.num_heads, q_len, kv_seq_len):
            raise ValueError(
                f"Attention weights should be of size {(bsz, self.num_heads, q_len, kv_seq_len)}, but is"
                f" {attn_weights.size()}"
            )

        if attention_mask is not None:
            if attention_mask.size() != (bsz, 1, q_len, kv_seq_len):
                raise ValueError(
                    f"Attention mask should be of size {(bsz, 1, q_len, kv_seq_len)}, but is {attention_mask.size()}"
                )
            attn_weights = attn_weights + attention_mask

        attn_weights = nn.functional.softmax(attn_weights, dim=-1)

        head_orders = torch.zeros([bsz, self.num_heads])
        chunks = torch.chunk(attn_weights, chunks=attn_weights.size(0), dim=0)

        for i, chunk in enumerate(chunks):
            chunk = chunk.squeeze(0)
            assert chunk.size() == (self.num_heads, q_len, kv_seq_len)
            head_orders[i] = self._sort_by_score(chunk)

        return head_orders.to(torch.int)
    
    def _sort_by_score(self, attn_weights):
        # attn_weights: [num_heads, q_len, kv_seq_len]
        lower_triangular = torch.tril(attn_weights.to(torch.float16), diagonal=-32)  # 2.0.1 -> tril BF16 Error
        sum_lower_triangular = lower_triangular.sum(dim=(1, 2))
        head_orders = torch.argsort(-sum_lower_triangular)

        return head_orders

    def forward(
        self,
        hidden_states: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.LongTensor] = None,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        output_attentions: bool = False,
        use_cache: bool = False,
        is_padded_inputs: Optional[bool] = False,
    ) -> Tuple[torch.Tensor, Optional[torch.Tensor], Optional[Tuple[torch.Tensor]]]:
        bsz, q_len, h_size = hidden_states.size()

        has_layer_past = past_key_value is not None

        if has_layer_past:
            past_kv = past_key_value[0]
            past_len = past_key_value[1]
        else:
            past_len = 0

        if self.config.pretraining_tp > 1:
            key_value_slicing = (self.num_key_value_heads * self.head_dim) // self.config.pretraining_tp
            query_slices = self.q_proj.weight.split(
                (self.num_heads * self.head_dim) // self.config.pretraining_tp, dim=0
            )
            key_slices = self.k_proj.weight.split(key_value_slicing, dim=0)
            value_slices = self.v_proj.weight.split(key_value_slicing, dim=0)

            q = [F.linear(hidden_states, query_slices[i]) for i in range(self.config.pretraining_tp)]
            q = torch.cat(q, dim=-1)

            k = [F.linear(hidden_states, key_slices[i]) for i in range(self.config.pretraining_tp)]
            k = torch.cat(k, dim=-1)

            v = [F.linear(hidden_states, value_slices[i]) for i in range(self.config.pretraining_tp)]
            v = torch.cat(v, dim=-1)

        else:
            q = self.q_proj(hidden_states)
            k = self.k_proj(hidden_states)
            v = self.v_proj(hidden_states) 

        q = q.view(bsz, q_len, self.num_heads, self.head_dim)
        k = k.view(bsz, q_len, self.num_key_value_heads, self.head_dim)
        v = v.view(bsz, q_len, self.num_key_value_heads, self.head_dim)

        if self.enable_head_metrics:
            kv_seq_len = q.size(1)
            new_k = k
            if has_layer_past:
                kv_seq_len = past_len + q.size(1)
                kv = torch.stack([k, v], 2)
                kv = repeat_kv(kv, self.num_key_value_groups)
                kv = torch.cat([past_kv[:, :past_len], kv], 1)
                new_k = kv[:, :, 0, :, :]

            head_order = self._head_wise_statistics(q, new_k, q_len, kv_seq_len, bsz, attention_mask)

            cos, sin = self.rotary_emb(v, seq_len=past_len, enable_head_metrics=self.enable_head_metrics)
            cos = cos.squeeze()[head_order, :, :]
            sin = sin.squeeze()[head_order, :, :]

        else:
            cos, sin = self.rotary_emb(v, seq_len=past_len)

        q, k = apply_rotary_pos_emb(q, k, cos, sin, position_ids)
        
        kv = torch.stack([k, v], 2)
        kv = repeat_kv(kv, self.num_key_value_groups)

        # Cache QKV values
        if has_layer_past:
            new_len = past_len + q.size(1)
            if new_len > past_kv.size(1):
                past_kv = torch.cat([past_kv, torch.empty(bsz, 256, 2, kv.size(3), kv.size(4), dtype=kv.dtype, device=kv.device)], 1)
            past_kv[:, past_len:new_len] = kv
            kv = past_kv[:, :new_len]
        else:
            past_kv = kv

        past_key_value = (past_kv, past_len + q.size(1)) if use_cache else None

        if is_padded_inputs:

            # varlen, ignore padding tokens, efficient for large batch with many paddings

            assert attention_mask is not None
            
            unpadded_kv, indices_k, cu_seqlens_k, max_seqlen_k = unpad_input(kv, attention_mask)
            unpadded_q, indices_q, cu_seqlens_q, max_seqlen_q = unpad_input(q, attention_mask[:, -q.size(1):])
            attn_outputs = flash_attn_varlen_kvpacked_func(
                unpadded_q, unpadded_kv, cu_seqlens_q, cu_seqlens_k, 
                max_seqlen_q, max_seqlen_k,
                dropout_p=0.0, softmax_scale=1.0/self.norm_factor, 
                causal=(not has_layer_past), return_attn_probs=output_attentions
            )

            attn_output = attn_outputs[0] if output_attentions else attn_outputs
            attn_output = pad_input(
                attn_output, indices_q, bsz, max_seqlen_q
            ).reshape(bsz, q_len, h_size)
            attn_weights = attn_outputs[2] if output_attentions else None
            
        else:

            # no padding tokens, more efficient
            
            attn_outputs = flash_attn_kvpacked_func(
                q, kv, dropout_p=0.0, softmax_scale=1.0/self.norm_factor, causal=(not has_layer_past), return_attn_probs=output_attentions)

            attn_output = attn_outputs[0] if output_attentions else attn_outputs
            attn_output = attn_output.reshape(bsz, q_len, h_size)
            attn_weights = attn_outputs[2] if output_attentions else None

        if self.config.pretraining_tp > 1:
            attn_output = attn_output.split(self.hidden_size // self.config.pretraining_tp, dim=2)
            o_proj_slices = self.o_proj.weight.split(self.hidden_size // self.config.pretraining_tp, dim=1)
            attn_output = sum([F.linear(attn_output[i], o_proj_slices[i]) for i in range(self.config.pretraining_tp)])
        else:
            attn_output = self.o_proj(attn_output)

        if not output_attentions:
            attn_weights = None

        return attn_output, attn_weights, past_key_value


class LlamaDecoderLayer(nn.Module):
    def __init__(self, config: LlamaConfig):
        super().__init__()
        self.hidden_size = config.hidden_size
        self.self_attn = LlamaAttention(config=config)
        self.mlp = LlamaMLP(config)
        self.input_layernorm = LlamaRMSNorm(config.hidden_size, eps=config.rms_norm_eps)
        self.post_attention_layernorm = LlamaRMSNorm(config.hidden_size, eps=config.rms_norm_eps)

    def forward(
        self,
        hidden_states: torch.Tensor,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.LongTensor] = None,
        past_key_value: Optional[Tuple[torch.Tensor]] = None,
        is_padded_inputs: Optional[bool] = False,
        output_attentions: Optional[bool] = False,
        use_cache: Optional[bool] = False,
    ) -> Tuple[torch.FloatTensor, Optional[Tuple[torch.FloatTensor, torch.FloatTensor]]]:
        """
        Args:
            hidden_states (`torch.FloatTensor`): input to the layer of shape `(batch, seq_len, embed_dim)`
            attention_mask (`torch.FloatTensor`, *optional*): attention mask of size
                `(batch, 1, tgt_len, src_len)` where padding elements are indicated by very large negative values.
            output_attentions (`bool`, *optional*):
                Whether or not to return the attentions tensors of all attention layers. See `attentions` under
                returned tensors for more detail.
            use_cache (`bool`, *optional*):
                If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding
                (see `past_key_values`).
            past_key_value (`Tuple(torch.FloatTensor)`, *optional*): cached past key and value projection states
        """

        residual = hidden_states

        hidden_states = self.input_layernorm(hidden_states)

        # Self Attention
        hidden_states, self_attn_weights, present_key_value = self.self_attn(
            hidden_states=hidden_states,
            attention_mask=attention_mask,
            position_ids=position_ids,
            past_key_value=past_key_value,
            output_attentions=output_attentions,
            use_cache=use_cache,
            is_padded_inputs=is_padded_inputs,
        )
        hidden_states = residual + hidden_states

        # Fully Connected
        residual = hidden_states
        hidden_states = self.post_attention_layernorm(hidden_states)
        hidden_states = self.mlp(hidden_states)
        hidden_states = residual + hidden_states

        outputs = (hidden_states,)

        if output_attentions:
            outputs += (self_attn_weights,)

        if use_cache:
            outputs += (present_key_value,)

        return outputs


LLAMA_START_DOCSTRING = r"""
    This model inherits from [`PreTrainedModel`]. Check the superclass documentation for the generic methods the
    library implements for all its model (such as downloading or saving, resizing the input embeddings, pruning heads
    etc.)

    This model is also a PyTorch [torch.nn.Module](https://pytorch.org/docs/stable/nn.html#torch.nn.Module) subclass.
    Use it as a regular PyTorch Module and refer to the PyTorch documentation for all matter related to general usage
    and behavior.

    Parameters:
        config ([`LlamaConfig`]):
            Model configuration class with all the parameters of the model. Initializing with a config file does not
            load the weights associated with the model, only the configuration. Check out the
            [`~PreTrainedModel.from_pretrained`] method to load the model weights.
"""


@add_start_docstrings(
    "The bare LLaMA Model outputting raw hidden-states without any specific head on top.",
    LLAMA_START_DOCSTRING,
)
class LlamaPreTrainedModel(PreTrainedModel):
    config_class = LlamaConfig
    base_model_prefix = "model"
    supports_gradient_checkpointing = True
    _no_split_modules = ["LlamaDecoderLayer"]
    _skip_keys_device_placement = "past_key_values"

    def _init_weights(self, module):
        std = self.config.initializer_range
        if isinstance(module, nn.Linear):
            module.weight.data.normal_(mean=0.0, std=std)
            if module.bias is not None:
                module.bias.data.zero_()
        elif isinstance(module, nn.Embedding):
            module.weight.data.normal_(mean=0.0, std=std)
            if module.padding_idx is not None:
                module.weight.data[module.padding_idx].zero_()

    def _set_gradient_checkpointing(self, module, value=False):
        if isinstance(module, LlamaModel):
            module.gradient_checkpointing = value


LLAMA_INPUTS_DOCSTRING = r"""
    Args:
        input_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`):
            Indices of input sequence tokens in the vocabulary. Padding will be ignored by default should you provide
            it.

            Indices can be obtained using [`AutoTokenizer`]. See [`PreTrainedTokenizer.encode`] and
            [`PreTrainedTokenizer.__call__`] for details.

            [What are input IDs?](../glossary#input-ids)
        attention_mask (`torch.Tensor` of shape `(batch_size, sequence_length)`, *optional*):
            Mask to avoid performing attention on padding token indices. Mask values selected in `[0, 1]`:

            - 1 for tokens that are **not masked**,
            - 0 for tokens that are **masked**.

            [What are attention masks?](../glossary#attention-mask)

            Indices can be obtained using [`AutoTokenizer`]. See [`PreTrainedTokenizer.encode`] and
            [`PreTrainedTokenizer.__call__`] for details.

            If `past_key_values` is used, optionally only the last `decoder_input_ids` have to be input (see
            `past_key_values`).

            If you want to change padding behavior, you should read [`modeling_opt._prepare_decoder_attention_mask`]
            and modify to your needs. See diagram 1 in [the paper](https://arxiv.org/abs/1910.13461) for more
            information on the default strategy.

            - 1 indicates the head is **not masked**,
            - 0 indicates the head is **masked**.
        position_ids (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
            Indices of positions of each input sequence tokens in the position embeddings. Selected in the range `[0,
            config.n_positions - 1]`.

            [What are position IDs?](../glossary#position-ids)
        past_key_values (`tuple(tuple(torch.FloatTensor))`, *optional*, returned when `use_cache=True` is passed or when `config.use_cache=True`):
            Tuple of `tuple(torch.FloatTensor)` of length `config.n_layers`, with each tuple having 2 tensors of shape
            `(batch_size, num_heads, sequence_length, embed_size_per_head)`) and 2 additional tensors of shape
            `(batch_size, num_heads, encoder_sequence_length, embed_size_per_head)`.

            Contains pre-computed hidden-states (key and values in the self-attention blocks and in the cross-attention
            blocks) that can be used (see `past_key_values` input) to speed up sequential decoding.

            If `past_key_values` are used, the user can optionally input only the last `decoder_input_ids` (those that
            don't have their past key value states given to this model) of shape `(batch_size, 1)` instead of all
            `decoder_input_ids` of shape `(batch_size, sequence_length)`.
        inputs_embeds (`torch.FloatTensor` of shape `(batch_size, sequence_length, hidden_size)`, *optional*):
            Optionally, instead of passing `input_ids` you can choose to directly pass an embedded representation. This
            is useful if you want more control over how to convert `input_ids` indices into associated vectors than the
            model's internal embedding lookup matrix.
        use_cache (`bool`, *optional*):
            If set to `True`, `past_key_values` key value states are returned and can be used to speed up decoding (see
            `past_key_values`).
        output_attentions (`bool`, *optional*):
            Whether or not to return the attentions tensors of all attention layers. See `attentions` under returned
            tensors for more detail.
        output_hidden_states (`bool`, *optional*):
            Whether or not to return the hidden states of all layers. See `hidden_states` under returned tensors for
            more detail.
        return_dict (`bool`, *optional*):
            Whether or not to return a [`~utils.ModelOutput`] instead of a plain tuple.
"""


@add_start_docstrings(
    "The bare LLaMA Model outputting raw hidden-states without any specific head on top.",
    LLAMA_START_DOCSTRING,
)
class LlamaModel(LlamaPreTrainedModel):
    """
    Transformer decoder consisting of *config.num_hidden_layers* layers. Each layer is a [`LlamaDecoderLayer`]

    Args:
        config: LlamaConfig
    """

    def __init__(self, config: LlamaConfig):
        super().__init__(config)
        self.padding_idx = config.pad_token_id
        self.vocab_size = config.vocab_size

        self.embed_tokens = nn.Embedding(config.vocab_size, config.hidden_size, self.padding_idx)
        self.layers = nn.ModuleList([LlamaDecoderLayer(config) for _ in range(config.num_hidden_layers)])
        self.norm = LlamaRMSNorm(config.hidden_size, eps=config.rms_norm_eps)

        self.gradient_checkpointing = False
        # Initialize weights and apply final processing
        self.post_init()

    def get_input_embeddings(self):
        return self.embed_tokens

    def set_input_embeddings(self, value):
        self.embed_tokens = value

    # Copied from transformers.models.bart.modeling_bart.BartDecoder._prepare_decoder_attention_mask
    def _prepare_decoder_attention_mask(self, attention_mask, input_shape, inputs_embeds, past_key_values_length):
        # create causal mask
        # [bsz, seq_len] -> [bsz, 1, tgt_seq_len, src_seq_len]
        combined_attention_mask = None
        if input_shape[-1] > 1:
            combined_attention_mask = _make_causal_mask(
                input_shape,
                inputs_embeds.dtype,
                device=inputs_embeds.device,
                past_key_values_length=past_key_values_length,
            )

        if attention_mask is not None:
            # [bsz, seq_len] -> [bsz, 1, tgt_seq_len, src_seq_len]
            expanded_attn_mask = _expand_mask(attention_mask, inputs_embeds.dtype, tgt_len=input_shape[-1]).to(
                inputs_embeds.device
            )
            combined_attention_mask = (
                expanded_attn_mask if combined_attention_mask is None else expanded_attn_mask + combined_attention_mask
            )

        return combined_attention_mask

    @add_start_docstrings_to_model_forward(LLAMA_INPUTS_DOCSTRING)
    def forward(
        self,
        input_ids: torch.LongTensor = None,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[List[torch.FloatTensor]] = None,
        inputs_embeds: Optional[torch.FloatTensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        is_padded_inputs: Optional[bool] = False,
    ) -> Union[Tuple, BaseModelOutputWithPast]:
        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        use_cache = use_cache if use_cache is not None else self.config.use_cache

        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        # retrieve input_ids and inputs_embeds
        if input_ids is not None and inputs_embeds is not None:
            raise ValueError("You cannot specify both decoder_input_ids and decoder_inputs_embeds at the same time")
        elif input_ids is not None:
            batch_size, seq_length = input_ids.shape
        elif inputs_embeds is not None:
            batch_size, seq_length, _ = inputs_embeds.shape
        else:
            raise ValueError("You have to specify either decoder_input_ids or decoder_inputs_embeds")

        seq_length_with_past = seq_length
        past_key_values_length = 0

        if past_key_values is not None:
            past_key_values_length = past_key_values[0][0].shape[2]
            seq_length_with_past = seq_length_with_past + past_key_values_length

        if position_ids is None:
            device = input_ids.device if input_ids is not None else inputs_embeds.device
            position_ids = torch.arange(
                past_key_values_length, seq_length + past_key_values_length, dtype=torch.long, device=device
            )
            position_ids = position_ids.unsqueeze(0).view(-1, seq_length)
        else:
            position_ids = position_ids.view(-1, seq_length).long()

        if inputs_embeds is None:
            inputs_embeds = self.embed_tokens(input_ids)

        # when is_padded_inputs=True, it will out of memory
        # if attention_mask is not None:
        #     attention_mask = self._prepare_decoder_attention_mask(
        #     attention_mask, (batch_size, seq_length), inputs_embeds, past_key_values_length
        # )

        hidden_states = inputs_embeds

        if self.gradient_checkpointing and self.training:
            if use_cache:
                logger.warning_once(
                    "`use_cache=True` is incompatible with gradient checkpointing. Setting `use_cache=False`..."
                )
                use_cache = False

        # decoder layers
        all_hidden_states = () if output_hidden_states else None
        all_self_attns = () if output_attentions else None
        next_decoder_cache = () if use_cache else None

        for idx, decoder_layer in enumerate(self.layers):
            if output_hidden_states:
                all_hidden_states += (hidden_states,)

            past_key_value = past_key_values[idx] if past_key_values is not None else None

            if self.gradient_checkpointing and self.training:

                def create_custom_forward(module):
                    def custom_forward(*inputs):
                        # None for past_key_value
                        return module(*inputs, output_attentions, None)

                    return custom_forward

                layer_outputs = torch.utils.checkpoint.checkpoint(
                    create_custom_forward(decoder_layer),
                    hidden_states,
                    attention_mask,
                    position_ids,
                    None,
                    is_padded_inputs
                )
            else:
                layer_outputs = decoder_layer(
                    hidden_states,
                    attention_mask=attention_mask,
                    position_ids=position_ids,
                    past_key_value=past_key_value,
                    output_attentions=output_attentions,
                    use_cache=use_cache,
                    is_padded_inputs=is_padded_inputs,
                )

            hidden_states = layer_outputs[0]

            if use_cache:
                next_decoder_cache += (layer_outputs[2 if output_attentions else 1],)

            if output_attentions:
                all_self_attns += (layer_outputs[1],)

        hidden_states = self.norm(hidden_states)

        # add hidden states from the last decoder layer
        if output_hidden_states:
            all_hidden_states += (hidden_states,)

        next_cache = next_decoder_cache if use_cache else None
        if not return_dict:
            return tuple(v for v in [hidden_states, next_cache, all_hidden_states, all_self_attns] if v is not None)
        return BaseModelOutputWithPast(
            last_hidden_state=hidden_states,
            past_key_values=next_cache,
            hidden_states=all_hidden_states,
            attentions=all_self_attns,
        )


class LlamaForCausalLM(LlamaPreTrainedModel):
    _tied_weights_keys = ["lm_head.weight"]

    def __init__(self, config):
        super().__init__(config)
        self.model = LlamaModel(config)
        self.vocab_size = config.vocab_size
        self.lm_head = nn.Linear(config.hidden_size, config.vocab_size, bias=False)

        # Initialize weights and apply final processing
        self.post_init()

    def get_input_embeddings(self):
        return self.model.embed_tokens

    def set_input_embeddings(self, value):
        self.model.embed_tokens = value

    def get_output_embeddings(self):
        return self.lm_head

    def set_output_embeddings(self, new_embeddings):
        self.lm_head = new_embeddings

    def set_decoder(self, decoder):
        self.model = decoder

    def get_decoder(self):
        return self.model

    @add_start_docstrings_to_model_forward(LLAMA_INPUTS_DOCSTRING)
    @replace_return_docstrings(output_type=CausalLMOutputWithPast, config_class=_CONFIG_FOR_DOC)
    def forward(
        self,
        input_ids: torch.LongTensor = None,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[List[torch.FloatTensor]] = None,
        inputs_embeds: Optional[torch.FloatTensor] = None,
        labels: Optional[torch.LongTensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
        is_padded_inputs: Optional[bool] = None,
    ) -> Union[Tuple, CausalLMOutputWithPast]:
        r"""
        Args:
            labels (`torch.LongTensor` of shape `(batch_size, sequence_length)`, *optional*):
                Labels for computing the masked language modeling loss. Indices should either be in `[0, ...,
                config.vocab_size]` or -100 (see `input_ids` docstring). Tokens with indices set to `-100` are ignored
                (masked), the loss is only computed for the tokens with labels in `[0, ..., config.vocab_size]`.

        Returns:

        Example:

        ```python
        >>> from transformers import AutoTokenizer, LlamaForCausalLM

        >>> model = LlamaForCausalLM.from_pretrained(PATH_TO_CONVERTED_WEIGHTS)
        >>> tokenizer = AutoTokenizer.from_pretrained(PATH_TO_CONVERTED_TOKENIZER)

        >>> prompt = "Hey, are you conscious? Can you talk to me?"
        >>> inputs = tokenizer(prompt, return_tensors="pt")

        >>> # Generate
        >>> generate_ids = model.generate(inputs.input_ids, max_length=30)
        >>> tokenizer.batch_decode(generate_ids, skip_special_tokens=True, clean_up_tokenization_spaces=False)[0]
        "Hey, are you conscious? Can you talk to me?\nI'm not conscious, but I can talk to you."
        ```"""

        output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
        output_hidden_states = (
            output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
        )
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        is_padded_inputs = ((attention_mask is not None) and (not attention_mask.all().item()))
        
        # decoder outputs consists of (dec_features, layer_state, dec_hidden, dec_attn)
        outputs = self.model(
            input_ids=input_ids,
            attention_mask=attention_mask,
            position_ids=position_ids,
            past_key_values=past_key_values,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
            is_padded_inputs=is_padded_inputs,
        )

        hidden_states = outputs[0]
        if self.config.pretraining_tp > 1:
            lm_head_slices = self.lm_head.weight.split(self.vocab_size // self.config.pretraining_tp, dim=0)
            logits = [F.linear(hidden_states, lm_head_slices[i]) for i in range(self.config.pretraining_tp)]
            logits = torch.cat(logits, dim=-1)
        else:
            logits = self.lm_head(hidden_states)
        logits = logits.float()

        loss = None
        if labels is not None:
            # Shift so that tokens < n predict n
            shift_logits = logits[..., :-1, :].contiguous()
            shift_labels = labels[..., 1:].contiguous()
            # Flatten the tokens
            loss_fct = CrossEntropyLoss()
            shift_logits = shift_logits.view(-1, self.config.vocab_size)
            shift_labels = shift_labels.view(-1)
            # Enable model parallelism
            shift_labels = shift_labels.to(shift_logits.device)
            loss = loss_fct(shift_logits, shift_labels)

        if not return_dict:
            output = (logits,) + outputs[1:]
            return (loss,) + output if loss is not None else output

        return CausalLMOutputWithPast(
            loss=loss,
            logits=logits,
            past_key_values=outputs.past_key_values,
            hidden_states=outputs.hidden_states,
            attentions=outputs.attentions,
        )

    def prepare_inputs_for_generation(
        self, input_ids, past_key_values=None, attention_mask=None, inputs_embeds=None, **kwargs
    ):
        if past_key_values:
            input_ids = input_ids[:, -1:]

        position_ids = kwargs.get("position_ids", None)
        if attention_mask is not None and position_ids is None:
            # create position_ids on the fly for batch generation
            position_ids = attention_mask.long().cumsum(-1) - 1
            position_ids.masked_fill_(attention_mask == 0, 1)
            if past_key_values:
                position_ids = position_ids[:, -1].unsqueeze(-1)

        # if `inputs_embeds` are passed, we only want to use them in the 1st generation step
        if inputs_embeds is not None and past_key_values is None:
            model_inputs = {"inputs_embeds": inputs_embeds}
        else:
            model_inputs = {"input_ids": input_ids}

        model_inputs.update(
            {
                "position_ids": position_ids,
                "past_key_values": past_key_values,
                "use_cache": kwargs.get("use_cache"),
                "attention_mask": attention_mask,
                "is_padded_inputs": ((attention_mask is not None) and (not attention_mask.all().item()))
            }
        )
        return model_inputs

    @staticmethod
    def _reorder_cache(past_key_values, beam_idx):
        reordered_past = ()
        for layer_past in past_key_values:
            reordered_past += (
                tuple(past_state.index_select(0, beam_idx.to(past_state.device)) for past_state in layer_past),
            )
        return reordered_past


@add_start_docstrings(
    """
    The LLaMa Model transformer with a sequence classification head on top (linear layer).

    [`LlamaForSequenceClassification`] uses the last token in order to do the classification, as other causal models
    (e.g. GPT-2) do.

    Since it does classification on the last token, it requires to know the position of the last token. If a
    `pad_token_id` is defined in the configuration, it finds the last token that is not a padding token in each row. If
    no `pad_token_id` is defined, it simply takes the last value in each row of the batch. Since it cannot guess the
    padding tokens when `inputs_embeds` are passed instead of `input_ids`, it does the same (take the last value in
    each row of the batch).
    """,
    LLAMA_START_DOCSTRING,
)
class LlamaForSequenceClassification(LlamaPreTrainedModel):
    def __init__(self, config):
        super().__init__(config)
        self.num_labels = config.num_labels
        self.model = LlamaModel(config)
        self.score = nn.Linear(config.hidden_size, self.num_labels, bias=False)

        # Initialize weights and apply final processing
        self.post_init()

    def get_input_embeddings(self):
        return self.model.embed_tokens

    def set_input_embeddings(self, value):
        self.model.embed_tokens = value

    @add_start_docstrings_to_model_forward(LLAMA_INPUTS_DOCSTRING)
    def forward(
        self,
        input_ids: torch.LongTensor = None,
        attention_mask: Optional[torch.Tensor] = None,
        position_ids: Optional[torch.LongTensor] = None,
        past_key_values: Optional[List[torch.FloatTensor]] = None,
        inputs_embeds: Optional[torch.FloatTensor] = None,
        labels: Optional[torch.LongTensor] = None,
        use_cache: Optional[bool] = None,
        output_attentions: Optional[bool] = None,
        output_hidden_states: Optional[bool] = None,
        return_dict: Optional[bool] = None,
    ) -> Union[Tuple, SequenceClassifierOutputWithPast]:
        r"""
        labels (`torch.LongTensor` of shape `(batch_size,)`, *optional*):
            Labels for computing the sequence classification/regression loss. Indices should be in `[0, ...,
            config.num_labels - 1]`. If `config.num_labels == 1` a regression loss is computed (Mean-Square loss), If
            `config.num_labels > 1` a classification loss is computed (Cross-Entropy).
        """
        return_dict = return_dict if return_dict is not None else self.config.use_return_dict

        transformer_outputs = self.model(
            input_ids,
            attention_mask=attention_mask,
            position_ids=position_ids,
            past_key_values=past_key_values,
            inputs_embeds=inputs_embeds,
            use_cache=use_cache,
            output_attentions=output_attentions,
            output_hidden_states=output_hidden_states,
            return_dict=return_dict,
        )
        hidden_states = transformer_outputs[0]
        logits = self.score(hidden_states)

        if input_ids is not None:
            batch_size = input_ids.shape[0]
        else:
            batch_size = inputs_embeds.shape[0]

        if self.config.pad_token_id is None and batch_size != 1:
            raise ValueError("Cannot handle batch sizes > 1 if no padding token is defined.")
        if self.config.pad_token_id is None:
            sequence_lengths = -1
        else:
            if input_ids is not None:
                sequence_lengths = (torch.ne(input_ids, self.config.pad_token_id).sum(-1) - 1).to(logits.device)
            else:
                sequence_lengths = -1

        pooled_logits = logits[torch.arange(batch_size, device=logits.device), sequence_lengths]

        loss = None
        if labels is not None:
            labels = labels.to(logits.device)
            if self.config.problem_type is None:
                if self.num_labels == 1:
                    self.config.problem_type = "regression"
                elif self.num_labels > 1 and (labels.dtype == torch.long or labels.dtype == torch.int):
                    self.config.problem_type = "single_label_classification"
                else:
                    self.config.problem_type = "multi_label_classification"

            if self.config.problem_type == "regression":
                loss_fct = MSELoss()
                if self.num_labels == 1:
                    loss = loss_fct(pooled_logits.squeeze(), labels.squeeze())
                else:
                    loss = loss_fct(pooled_logits, labels)
            elif self.config.problem_type == "single_label_classification":
                loss_fct = CrossEntropyLoss()
                loss = loss_fct(pooled_logits.view(-1, self.num_labels), labels.view(-1))
            elif self.config.problem_type == "multi_label_classification":
                loss_fct = BCEWithLogitsLoss()
                loss = loss_fct(pooled_logits, labels)
        if not return_dict:
            output = (pooled_logits,) + transformer_outputs[1:]
            return ((loss,) + output) if loss is not None else output

        return SequenceClassifierOutputWithPast(
            loss=loss,
            logits=pooled_logits,
            past_key_values=transformer_outputs.past_key_values,
            hidden_states=transformer_outputs.hidden_states,
            attentions=transformer_outputs.attentions,
        )
